/****************************************************************************
 * crypto/softmodule.c
 *
 *   Copyright (C) 2018 Sebastien Lorquet. All rights reserved.
 *   Author:  Sebastien Lorquet <sebastien@lorquet.fr>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name NuttX nor the names of its contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

/****************************************************************************
 * Included Files
 ****************************************************************************/

#include <nuttx/config.h>

#include <stdint.h>
#include <stdbool.h>

#include <errno.h>

#include <nuttx/kmalloc.h>
#include <nuttx/semaphore.h>
#include <nuttx/fs/fs.h>
#include <nuttx/crypto/module.h>
#include <nuttx/crypto/manager.h>

static FAR void* softcrypto_sessioncreate(void);
static int softcrypto_sessionfree(FAR void* session);
static int softcrypto_authenticate(FAR void *session, uint32_t step,
                                   uint32_t cmdlen, FAR uint8_t *cmd,
                                   uint32_t rsplen, uint8_t rsp);
static int softcrypto_keycount(FAR void *session);
static int softcrypto_keyinfo(FAR void *session, int index, FAR uint32_t *id,
                              FAR uint32_t *flags, FAR uint32_t *length);
static int softcrypto_keycreate(FAR void *session, uint32_t id, uint32_t flags,
                                uint32_t length);
static int softcrypto_keydelete(FAR void *session, uint32_t id);
static int softcrypto_keygen(FAR void *session, uint32_t id, uint32_t rngalg);
static int softcrypto_keyupdate(FAR void *session, uint32_t id,
                                uint32_t component, uint32_t unwrapalg,
                                uint32_t len, FAR uint8_t *value);
static int softcrypto_keyread(FAR void *session, uint32_t id,
                              uint32_t component, uint32_t buflen,
                              FAR uint8_t *buffer);
static int softcrypto_keycopy(FAR void *session, uint32_t dest, uint32_t src);
static bool softcrypto_alg_supported(FAR void *session, uint32_t algid);
static int softcrypto_alg_init(FAR void *session, uint32_t algid, uint32_t mode,
                               uint32_t keyid);
static int softcrypto_alg_update(FAR void *session, uint32_t inlen,
                                 FAR uint8_t *indata, uint32_t outlen,
                                 FAR uint8_t *outdata);
static int softcrypto_alg_finish(FAR void *session, uint32_t inlen,
                                 FAR uint8_t *indata, uint32_t outlen,
                                 FAR uint8_t *outdata);

struct cryptomodule_ops_s softmodule_ops =
{
  softcrypto_sessioncreate,
  softcrypto_sessionfree,
  softcrypto_authenticate,
  softcrypto_keycount,
  softcrypto_keyinfo,
  softcrypto_keycreate,
  softcrypto_keydelete,
  softcrypto_keygen,
  softcrypto_keyupdate,
  softcrypto_keyread,
  softcrypto_keycopy,
  softcrypto_alg_supported,
  softcrypto_alg_init,
  softcrypto_alg_update,
  softcrypto_alg_finish,
};

struct cryptomodule_s softmodule =
{
    "SOFTCRYPTO",
    &softmodule_ops
};

static FAR void* softcrypto_sessioncreate(void)
{
  return NULL;
}

static int softcrypto_sessionfree(FAR void* session)
{
  return 0;
}

static int softcrypto_authenticate(FAR void *session, uint32_t step,
                                   uint32_t cmdlen, FAR uint8_t *cmd,
                                   uint32_t rsplen, uint8_t rsp)
{
  return 0;
}


static int softcrypto_keycount(FAR void *session)
{
  return 0;
}

static int softcrypto_keyinfo(FAR void *session, int index, FAR uint32_t *id,
                              FAR uint32_t *flags, FAR uint32_t *length)
{
  return 0;
}

static int softcrypto_keycreate(FAR void *session, uint32_t id, uint32_t flags,
                                uint32_t length)
{
  return 0;
}

static int softcrypto_keydelete(FAR void *session, uint32_t id)
{
  return 0;
}

static int softcrypto_keygen(FAR void *session, uint32_t id, uint32_t rngalg)
{
  return 0;
}

static int softcrypto_keyupdate(FAR void *session, uint32_t id,
                                uint32_t component, uint32_t unwrapalg,
                                uint32_t len, FAR uint8_t *value)
{
  return 0;
}

static int softcrypto_keyread(FAR void *session, uint32_t id,
                              uint32_t component, uint32_t buflen,
                              FAR uint8_t *buffer)
{
  return 0;
}

static int softcrypto_keycopy(FAR void *session, uint32_t dest, uint32_t src)
{
  return 0;
}

static bool softcrypto_alg_supported(FAR void *session, uint32_t algid)
{
  return 0;
}

static int softcrypto_alg_init(FAR void *session, uint32_t algid, uint32_t mode,
                               uint32_t keyid)
{
  return 0;
}

static int softcrypto_alg_update(FAR void *session, uint32_t inlen,
                                 FAR uint8_t *indata, uint32_t outlen,
                                 FAR uint8_t *outdata)
{
  return 0;
}

static int softcrypto_alg_finish(FAR void *session, uint32_t inlen,
                                 FAR uint8_t *indata, uint32_t outlen,
                                 FAR uint8_t *outdata)
{
  return 0;
}


int softcrypto_register(void)
{
  return cryptomod_register(&softmodule);
}

